<?php

namespace Drupal\captchetat;

use Drupal\Core\Config\ImmutableConfig;

/**
 * Helper methods for calling CaptchEtat APIs.
 */
class CaptchetatApiHelper {

  /**
   * Returns if API is up or down.
   *
   * @param string $token
   *   CaptchEtat API access token.
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   CaptchEtat module configuration.
   *
   * @return bool|string
   *   The call to API curl_exec response string.
   */
  public static function healthCheck(
    string $token,
    ImmutableConfig $config,
  ): bool|string {

    $url = $config->get('api_url') . '/piste/captcha/healthcheck';

    $ch = curl_init($url);
    $headers = [
      "Authorization: Bearer $token",
    ];

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

    $response = curl_exec($ch);

    if (curl_errno($ch)) {
      echo 'Curl error: ' . curl_error($ch);
    }

    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
    curl_close($ch);

    return $response;
  }

  /**
   * Returns the captchetat token.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   CaptchEtat module configuration.
   *
   * @return string
   *   token provided by API CaptchEtat.
   */
  public static function generateApiToken(
    ImmutableConfig $config,
  ): string {

    $client_id = $config->get('client_id');
    $client_secret = $config->get('client_secret');
    $url = $config->get('oauth_url') . '/api/oauth/token';

    $data = [
      'grant_type' => 'client_credentials',
      'client_id' => $client_id,
      'client_secret' => $client_secret,
      'scope' => 'piste.captchetat',
    ];
    $ch = curl_init($url);

    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/x-www-form-urlencoded']);
    curl_setopt($ch, CURLOPT_TIMEOUT, 3600);

    $response = curl_exec($ch);
    $result = json_decode($response);
    $token = $result->access_token;

    if (curl_errno($ch)) {
      echo 'Curl error: ' . curl_error($ch);
    }

    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
    curl_close($ch);

    return $token;
  }

  /**
   * Validates the user submission.
   *
   * @param string $captcha_id
   *   The captcha ID.
   * @param string $user_entered_captcha_code
   *   The user entered captcha code.
   * @param string $token
   *   CaptchEtat API access token.
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   CaptchEtat module configuration.
   *
   * @return bool|string
   *   The call to API curl_exec response string.
   */
  public static function validationCaptcha(
    string $captcha_id,
    string $user_entered_captcha_code,
    string $token,
    ImmutableConfig $config,
  ): bool|string {

    $url = $config->get('api_url') . '/piste/captcha/valider-captcha';

    $data = [
      'id' => $captcha_id,
      'code' => $user_entered_captcha_code,
    ];

    $jsonData = json_encode($data);

    $headers = [
      "Authorization: Bearer $token",
      'Content-Type: application/json',
    ];

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

    $response = curl_exec($ch);

    if (curl_errno($ch)) {
      echo 'Curl error: ' . curl_error($ch);
    }

    curl_close($ch);

    return $response;
  }

}
