<?php

namespace Drupal\captchetat\Controller;

use Drupal\captchetat\CaptchetatApiHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use GuzzleHttp\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * This controller provides different connections to API CaptchEtat.
 */
class CaptchetatController extends ControllerBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * HTTP client service.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * Constructs a new CaptchetatController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \GuzzleHttp\Client $http_client
   *   HTTP client service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Client $http_client) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client')
    );
  }

  /**
   * Returns if API is up or down.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response.
   */
  public function healthCheck() {

    $token = $this->generateApiToken();
    $config = $this->configFactory->get('captchetat.settings');

    $response = CaptchetatApiHelper::healthCheck($token, $config);

    return new Response($response);
  }

  /**
   * Returns the captchetat token.
   *
   * @return string
   *   token provided by API CaptchEtat.
   */
  public function generateApiToken() {
    $config = $this->configFactory->get('captchetat.settings');
    return CaptchetatApiHelper::generateApiToken($config);
  }

  /**
   * Gets Captcha from API Captchetat.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response for the CaptchEtat element.
   */
  public function getCaptcha(Request $request) {
    $token = $this->generateApiToken();
    $config = $this->configFactory->get('captchetat.settings');
    $request_uri = $request->getRequestUri();
    $query_params = explode('?', $request_uri)[1];
    $url = $config->get('api_url') . '/piste/captcha/simple-captcha-endpoint?' . $query_params;
    try {
      $headers = ['Authorization' => "Bearer $token"];
      $response = $this->httpClient->get($url, ['headers' => $headers]);
      $content_type = $response->getHeaders()['content-type'][0];
      return new Response(
        $response->getBody()->getContents(),
        $response->getStatusCode(),
        ['Content-Type' => $content_type]
      );
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('captchetat')->error($e->getMessage());
      return new Response('error');
    }
  }

  /**
   * Validates the user submission.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   HTTP Response for validation whether the user entered the correct code.
   */
  public function validationCaptcha(Request $request) {

    $payload = $request->getContent();
    $decoded_payload = json_decode($payload, TRUE);
    $captcha_id = $decoded_payload['captchaId'];
    $user_entered_captcha_code = $decoded_payload['userEnteredCaptchaCode'];

    $token = $this->generateApiToken();

    $config = $this->configFactory->get('captchetat.settings');

    $response = CaptchetatApiHelper::validationCaptcha(
      $captcha_id, $user_entered_captcha_code, $token, $config);

    return new Response($response);
  }

}
